package db;

import java.io.Serializable;
import java.text.ParseException;
import java.util.HashMap;
import java.util.Vector;

/// 测点+单个对应的时刻的数值记录
public class DBVal implements Serializable {
    private static final long serialVersionUID = 1L;

    /// 测点
    private Point point;
    /// 数值型实时UTC时间，单位纳秒
    private long utcTime;
    /// 数值型实时数值
    private double value;
    private ValueStatus valueStatus = ValueStatus.VALID;

    public DBVal(Point point, long utcTime, double value) {
        this.point = point;
        this.utcTime = utcTime;
        this.value = value;
    }

    public DBVal(Point point, long utcTime, double value,
                 ValueStatus valueStatus) {
        this.point = point;
        this.utcTime = utcTime;
        this.value = value;
        this.valueStatus = valueStatus;
    }

    public DBVal(Point point, long utcTime, int value) {
        this.point = point;
        this.utcTime = utcTime;
        this.value = value;
    }

    public DBVal(Point point, long utcTime) {
        this.point = point;
        this.utcTime = utcTime;
    }

    public DBVal() {
    }

    public String getMetricName() {
        if (this.point == null) return "NULL";
        return this.point.getMetricName();
    }

    public void setMetricName(String metricName) {
        this.point.setMetricName(metricName);
    }

    public String getPointName() {
        if (this.point == null) return "NULL";
        return this.point.getPointName();
    }

    public void setPointName(String pointName) {
        this.point.setPointName(pointName);
    }

    /*
     * 返回时间戳，单位为秒
     */
    public long getUtcTime() {
        return utcTime / 1000_000_000;
    }

    /*
     * 设置时间戳，单位为秒
     */
    public void setUtcTime(long utcTime) {
        this.utcTime = utcTime * 1000_000_000;
    }

    /// 返回时间戳，单位为毫秒
    public long getUtcTimeMilliSeconds() {
        return utcTime / 1000_000;
    }

    /// 设置时间戳，单位为毫秒
    public void setUtcTimeMilliSeconds(long utcTime) {
        this.utcTime = utcTime * 1000_000;
    }

    /*
     * 返回时间戳，单位为纳秒
     */
    public long getUtcTimeNanoSeconds() {
        return utcTime;
    }

    /*
     * 设置时间戳，单位为纳秒
     */
    public void setUtcTimeNanoSeconds(long utcTime) {
        this.utcTime = utcTime;
    }

    public double getValue() {
        return value;
    }

    public void setValue(double value) {
        this.value = value;
    }

    public int getIntValue() {
        return (int) this.value;
    }

    public long getLongValue() {
        return (long) this.value;
    }

    public Point getPoint() {
        return point;
    }

    public void setPoint(Point point) {
        this.point = point;
    }

    public HashMap<String, String> getTags() {
        if (this.point == null) return null;
        return this.point.getTags();
    }

    public void setTags(HashMap<String, String> tags) {
        this.point.setTags(tags);
    }

    public String getFinalMetricName(String metricNamePrefix) {
        return metricNamePrefix + this.point.getMetricName();
    }

    public boolean isValueValid() {
        return this.valueStatus.equals(ValueStatus.VALID);
    }

    public void setValueStatusInValid() {
        this.valueStatus = ValueStatus.INVALID;
    }

    public ValueStatus getValueStatus() {
        return valueStatus;
    }

    public void setValueStatus(ValueStatus valueStatus) {
        this.valueStatus = valueStatus;
    }

    public String toString() {
        try {
            return String.format("[metricName:%s tags:%s utcTime:%d(ms) date:%s value:%f valueStatus:%s]\n",
                    this.getMetricName(), this.getTags(), this.utcTime,
                    Util.UTCNanosecondsToDateString(this.utcTime),
                    this.value,
                    this.valueStatus.toString());
        } catch (ParseException e) {
            e.printStackTrace();
            return "";
        }
    }

    public enum ValueStatus {
        VALID, INVALID
    }
}

