package db;

import java.io.Serializable;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

/// 测点+多个对应的时刻的数值记录
public class PointVals implements Serializable {
    private static final long serialVersionUID = 1L;

    /// 测点
    private Point point;
    /// 数值个数
    private int count;
    /// 数值型实时UTC时间，单位纳秒
    private long[] utcTimes;
    /// 数值型实时数值
    private double[] values;
    private int beginIndex = 0;

    public PointVals(Point point, int count, long[] utcTimes, double[] values) {
        this.point = point;
        this.count = count;
        this.utcTimes = utcTimes;
        this.values = values;
    }

    public PointVals() {
    }

    public String getMetricName() {
        return this.point.getMetricName();
    }

    public void setMetricName(String metricName) {
        this.point.setMetricName(metricName);
    }

    public Point getPoint() {
        return this.point;
    }

    public void setPoint(Point point) {
        this.point = point;
    }

    public String getPointName() {
        return this.point.getPointName();
    }

    public void setPointName(String pointName) {
        this.point.setPointName(pointName);
    }

    public void setBeginIndex(int beginIndex) {
        this.beginIndex = beginIndex;
    }

    public HashMap<String, String> getTags() {
        return this.point.getTags();
    }

    public void setTags(HashMap<String, String> tags) {
        this.point.setTags(tags);
    }

    public int getCount() {
        return count;
    }

    public void setCount(int count) {
        this.count = count;
    }

    public long getUtcTime(int index) {
        return this.utcTimes[beginIndex + index];
    }

    public double getValue(int index) {
        return this.values[beginIndex + index];
    }

    public String getFinalMetricName(String metricNamePrefix) {
        return metricNamePrefix + this.getMetricName();
    }

    public boolean isEmpty() {
        return count == 0;
    }

    public PointVals subPointVals(int begin, int end) {
        int count = end - begin;
        PointVals resultPointVals = new PointVals(this.point, count, this.utcTimes, this.values);
        resultPointVals.setBeginIndex(begin);
        return resultPointVals;
    }

    public DBVal getDBVal(int index) {
        if (index < 0 || index >= count) {
            return null;
        }
        return new DBVal(this.getPoint(), this.getUtcTime(index), this.getValue(index));
    }

    public List<DBVal> getDBValList() {
        List<DBVal> dbValList = new ArrayList<>();
        for (int i = 0; i < count; i++) {
            dbValList.add(this.getDBVal(i));
        }
        return dbValList;
    }
}
