package db;

import java.text.ParseException;
import java.util.List;

/**
 * 功能描述
 *
 * @author: scott
 * @date: 2022年09月29日 12:09 PM
 */
public class RangeQueryResult {
    Point point;
    long start;
    long end;
    long step;
    long lookBack;
    int count;
    List<DBVal> dbValList;
    boolean success = true;

    public RangeQueryResult(Point point, long start, long end,
                            long step, long lookBack,
                            List<DBVal> dbValList) {
        this.point = point;
        this.start = start;
        this.end = end;
        this.step = step;
        this.lookBack = lookBack;
        this.dbValList = dbValList;
//        this.count = (int)((end-start)/step);
        if (step == 0) {
            this.count = dbValList.size();
        } else {
            this.count = (int) ((end - start + step - 1) / step);
        }
    }

    public List<DBVal> getDbValList() {
        if (dbValList == null || dbValList.isEmpty()) {
            return DBUtil.invalidDBValList(point, start, step, count);
        }
        return dbValList;
    }

    public Point getPoint() {
        return point;
    }

    public long getStart() {
        return start;
    }

    public long getEnd() {
        return end;
    }

    public long getStep() {
        return step;
    }

    public long getLookBack() {
        return lookBack;
    }

    public boolean isSuccess() {
        return success;
    }

    public void setSuccess(boolean success) {
        this.success = success;
    }

    public int getCount() {
        return count;
    }

    @Override
    public String toString() {
        try {
            return "RangeQueryResult{" +
                    "point=" + point +
                    ", start=" + start + String.format("(%s)", Util.UTCNanosecondsToDateString(start)) +
                    ", end=" + end + String.format("(%s)", Util.UTCNanosecondsToDateString(end)) +
                    ", step=" + step +
                    ", lookBack=" + lookBack +
                    ", count=" + count +
                    ", dbValList=" + dbValList +
                    ", success=" + success +
                    '}';
        } catch (Exception e) {
            e.printStackTrace();
        }
        return "";
    }
}
